"""Doc of bioat.

"""
from __future__ import annotations
import sys

from pysam import __version__ as __pysam_version__
from pysamstats import __version__ as __pysamstats_version__

from bioat._version import version as __version__
from bioat._version import author as __author__
from bioat._version import email as __email__
from bioat._version import upgrade_date as __upgrade_date__
from bioat._logging import set_logging_level

__doc_format__ = "restructuredtext"
__name__ = "bioat"



"""
FASTQ 读取
FASTA 读取
...

load_fastx
load_fastx_generator

"""
import gzip


def load_fastx(file: str) -> list:
    """

    :param str file: path of input <fastq | fastq.gz | fasta | fasta.gz>
    :return: a list for all reads:
        i.e. [['header', 'seq', 'info', 'quality'], [...],[...]]
    :rtype: list
    """
    f = open(file, 'rt') if not '.gz' in file else gzip.open(file, 'rt')
    # FASTQ @  FASTA >
    symbol = f.read(1)
    # print(symbol)
    f.close()
    # 全部载入内存就好，当文件很小的时候
    f = open(file, 'rt') if not '.gz' in file else gzip.open(file, 'rt')

    if symbol == '@':
        # FASTQ
        # print('is fastq')
        ls = []
        read = []
        raw_info = [i.rstrip() for i in f.readlines()]
        line_fix = None  # 为了 fix quality 中开头为@的情况

        # for line in tqdm(raw_info):
        for line in raw_info:
            if line.startswith('@'):
                n = len(read)
                if n == 0:
                    # 第一次循环开始
                    read.append(line)  # add header
                elif n == 4:
                    # 后面的循环开始
                    ls.append(read)
                    read = []
                    read.append(line)  # add header
                elif n == 3:
                    # try to add quality line
                    """Fix a bug.
                    如果第 4 行是@开头,先正常操作一行 read 的 append,再判断下一行是否是@开头,如果不是,文件损坏,如果是,没问题继续
                    ['@Beta12AdemL1C001R00100001768/1', 
                    'ATCCCCGTATCTTCACCCCACCACAAACTATTAGCTTTAGA', 
                    '+']
                    '@@IIEIBCE>IC<IBIIIIEAIEIEB<IDECCD6ICBCED<'
                    """
                    # print(read)
                    # print(line)
                    if line_fix:
                        # 遇到需要fix的下一次循环
                        read.append(line_fix)
                        ls.append(read)
                        read = []
                        read.append(line)
                        line_fix = None
                    else:
                        # 第一次遇到需要 fix 的 line
                        line_fix = line
                else:
                    f.close()
                    raise ValueError('The file may be incomplete!')
            else:
                read.append(line)  # add other three lines: seq, info, quality

        ls.append(read)
        f.close()
        return ls


    elif symbol == '>':
        # FASTA
        # print('is fasta!')
        ls = []
        read = []
        seq = ''
        raw_info = [i.rstrip() for i in f.readlines()]
        # print(raw_info)

        for line in raw_info:
            if line.startswith('>'):
                # 读取 header！
                n = len(read)

                if n == 0:
                    # 第一次循环
                    read.append(line)
                elif n == 1:
                    # 已经有一个 header 了！现在缺 seq
                    read.append(seq)  # add seq line
                    ls.append(read)
                    read = []  # 重置 read 这个 list
                    read.append(line)
                    seq = ''  # 重置 seq 这个 str
                else:
                    f.close()
                    raise ValueError('The file may be incomplete!')
            else:
                # 读取并添加 seq
                seq += line

        read.append(seq)
        ls.append(read)
        return ls
    else:
        raise ValueError('Input line one must starts with @ for FQ or > for FA!')
    # f.close()

# 当文件很大的时候，用生成器函数，产生一个迭代器来进行文件读取
def load_fastx_generator(file):
    """

    :param str file: path of input <fastq | fastq.gz | fasta | fasta.gz>
    :return: a generator for all reads:
        i.e. print(next(obj)) -> ['header', 'seq', 'info', 'quality']
    :rtype: generator
    """
    f = open(file, 'rt') if not '.gz' in file else gzip.open(file, 'rt')
    # FASTQ @  FASTA >
    symbol = f.read(1)
    # print(symbol)
    f.close()
    # 全部载入内存就好，当文件很小的时候
    f = open(file, 'rt') if not '.gz' in file else gzip.open(file, 'rt')

    if symbol == '@':
        # FASTQ
        # print('is fastq!')
        # ls = []
        read = []
        line = f.readline().rstrip()

        while True:
            if not line:
                break
            else:
                if line.startswith('@'):
                    n = len(read)
                    if n == 0:
                        # read == []
                        # 第一次循环开始
                        read.append(line)  # add header!
                        line = f.readline().rstrip()
                    elif n == 4:
                        # read == ['header', 'seq', 'info', 'quality']
                        # ls.append(read)
                        yield read
                        read = []
                        read.append(line)
                        line = f.readline().rstrip()
                    elif n == 3:
                        # TODO
                        """
                        @@IIEIBCE>IC<IBIIIIEAIEIEB<IDECCD6 # line! 期望它是 header！现在它是 quality
                        [
                            '@Beta12AdemL1C001R00100001768/1', 
                            'ATCCCCGTATCTTCACCCCACCACAAACTATTAG', 
                            '+',
                        ]'@@IIEIBCE>IC<IBIIIIEAIEIEB<IDECCD6'

                        """
                        read.append(line)
                        line = f.readline().rstrip()

                        if not line.startswith('@'):
                            raise ValueError('The file may be incomplete!')
                    else:
                        f.close()
                        raise ValueError('The file may be incomplete!')

                    # header line!
                    # read.append(line)  # add header !
                else:
                    # not header line!
                    read.append(line)
                    line = f.readline().rstrip()

        # ls.append(read)
        yield read
        f.close()
        # return ls


    elif symbol == '>':
        # FASTA
        # print('is fasta!')
        ls = []
        read = []
        seq = ''
        # raw_info = [i.rstrip() for i in f.readlines()]
        line = f.readline().rstrip()
        # print(raw_info)

        while True:
            if not line:
                break
            else:
                if line.startswith('>'):
                    # 读取 header！
                    n = len(read)

                    if n == 0:
                        # 第一次循环
                        read.append(line)
                        line = f.readline().rstrip()
                    elif n == 1:
                        # 已经有一个 header 了！现在缺 seq
                        read.append(seq)  # add seq line
                        # ls.append(read)
                        yield read
                        read = []  # 重置 read 这个 list
                        read.append(line)
                        line = f.readline().rstrip()
                        seq = ''  # 重置 seq 这个 str
                    else:
                        f.close()
                        raise ValueError('The file may be incomplete!')
                else:
                    # 读取并添加 seq
                    seq += line
                    line = f.readline().rstrip()

        read.append(seq)
        # ls.append(read)
        yield read
        # return ls
    else:
        f.close()
        raise ValueError('Input line one must starts with @ for FQ or > for FA!')
    f.close()

if __name__ == '__main__':
    pass


import fire
from bioat import __name__ as name
from bioat import __version__ as version
from bioat import __author__ as author
from bioat import __email__ as email
from bioat import __upgrade_date__ as upgrade_date
from bioat import __pysam_version__
from bioat import __pysamstats_version__


about = f"""
    - bioat version:
        {version}
    - repository address:
        https://github.com/hermanzhaozzzz/BioinformaticAnalysisTools
    - author:
        - name: {author}
        - email: {email}
    - last update:
        - {upgrade_date}
    - key dependencies:
        - pysam == {__pysam_version__}
        - pysamstats == {__pysamstats_version__}
"""

class Cli(object):
    """Cli interface of python package <bioat>

    - bioat is a commandline tool and a python package, which is short for
        Bioinformatic Analysis Tools.
    - bioat has many subcommand to deal with different bio-format:
        BED, BAM, FASTA, FASTQ, VCF, et al.
    """

    def add(self, x, y):
        return x + y

    def multiply(self, x, y):
        return x * y
    def about(self):
        """Print information about <bioat>
        """
        return about


def main() -> int:
    calculator = Cli()
    fire.Fire(calculator, name=name)


if __name__ == "__main__":
    main()



import pandas as pd
from sys import stdout
from fire import Fire


class MergeTableCli():
    """A simple tool to merge tables from different sample.

    """

    def from_csv(
            self,
            inputs: str,
            tags: str,
            output=stdout.name,
            output_fmt='csv',
            input_header=True,
            output_header=True):
        """Merge from csv files

        Params
        :param inputs: csv files joint by comma
        :param tags: tags for each file
        :param output: output file name
        :param output_fmt: define the output format
        :param input_header: True or False
        :param output_header: True of False
        """
        return merge_table(
            inputs=inputs,
            tags=tags,
            output=output,
            input_fmt='csv',
            output_fmt=output_fmt,
            input_header=input_header,
            output_header=output_header)

    def from_tsv(
            self,
            inputs: str,
            tags: str,
            output=stdout.name,
            output_fmt='tsv',
            input_header=True,
            output_header=True):
        """Merge from tsv files
        Params
        :param inputs: csv files joint by comma
        :param tags: tags for each file
        :param output: output file name
        :param output_fmt: define the output format
        :param input_header: True or False
        :param output_header: True of False
        """
        return merge_table(
            inputs=inputs,
            tags=tags,
            output=output,
            input_fmt='tsv',
            output_fmt=output_fmt,
            input_header=input_header,
            output_header=output_header)


def deal_with_comma_params(params) -> list:
    params = list(params) if isinstance(params, tuple) else params.split(',')
    return [i.strip() for i in params]


def merge_table(
        inputs,
        tags,
        output,
        input_fmt,
        output_fmt,
        input_header=True,
        output_header=True):
    # fix params
    inputs = deal_with_comma_params(inputs)
    tags = deal_with_comma_params(tags)
    input_header = 0 if input_header else None
    output = stdout if output == stdout.name else output
    dt_sep = {'csv': ',', 'tsv': '\t'}

    df = pd.DataFrame()

    for input, tag in zip(inputs, tags):
        _df = pd.read_csv(
            input,
            sep=dt_sep[input_fmt],
            header=input_header,
            index_col=None)
        _df.insert(0, column='<sample>', value=tag)
        df = pd.concat([df, _df], axis=0, ignore_index=True)

    df.to_csv(output, sep=dt_sep[output_fmt], header=output_header, index=None, )


def main():
    Fire(MergeTableCli, name='merge-table')


if __name__ == "__main__":
    main()



import os
import gzip
from sys import stdout
from bioat._loader import load_fastx, load_fastx_generator


def main(input, output=stdout.name):
    # fix params
    output = stdout if output == stdout.name else output

    if isinstance(output, str):
        f = open(output, 'wt') if not output.endswith('.gz') else gzip.open(output, 'wt')
    else:
        f = output

    fa = load_fastx_generator(input)


    for read in fa:
        header, seq = read

        if seq.upper().__contains__('N'):
            continue
        else:
            f.write(f'{header}\n{seq}\n')

    f.close()


if __name__ == "__main__":
    main(input='../../../data/random_l-180_n-100_hg38_minus-test.fa', output='/Users/zhaohuanan/Downloads/testout.fa.gz')



from sys import stdout
from fire import Fire
from bioat import set_logging_level
from bioat.fasta._filter_n import main as filter_n


class Cli():
    """Fasta tools
    """

    def filter_n(self, input, output=stdout.name):
        """filter reads contains N

        Params
        :param str input: path of input <fasta|fasta.gz>
        :param str output: path of output <fasta|fasta.gz>
        """
        return filter_n(input, output)


def main() -> int:
    cli = Cli()
    Fire(cli, name='bioat_fastatools')


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['bioat', 'bioat.fasta', 'bioat.table']

package_data = \
{'': ['*']}

install_requires = \
['fire>=0.4.0,<0.5.0',
 'numpy>=1.23.3,<2.0.0',
 'pandas>=1.5.0,<2.0.0',
 'pip>=22.2.2,<23.0.0',
 'pysam==0.19.1',
 'tqdm>=4.64.1,<5.0.0']

entry_points = \
{'console_scripts': ['bioat = bioat._cli:main',
                     'bioat-table-merge-table = bioat.table.merge_table:main',
                     'bioat_fastatools = bioat.fasta._cli:main']}

setup_kwargs = {
    'name': 'bioat',
    'version': '0.1.1.5',
    'description': "Bioinformatic toolkit with python (It's still under development!)",
    'long_description': '# BioinformaticAnalysisTools\n\n## Introduction\nA bioinformatic python toolkit accelerated with rust!\n\n- Author: Hua-nan ZHAO @Tsinghua University\n- E-Mail: hermanzhaozzzz@gmail.com\n\n## Installation\n```shell\npip install bioat\n```\n\n## unit testing\n```shell\ncd tests\npython -m pytest\n# or\npoetry run pytest\n```\n## usage\n\n\n## history\n\n\n\n\n',
    'author': 'Herman Zhao',
    'author_email': 'hermanzhaozzzz@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/hermanzhaozzzz/BioinformaticAnalysisTools',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
